/*
	Minimalistic FAT32 library
	Copyright (C) 2009  Avelino Herrera Morales

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef  __FAT32_H__
#define  __FAT32_H__


#include <stdint.h>


typedef struct {
	uint32_t firstLBA;
	uint32_t lastLBA;
	uint32_t fatBeginLBA;
	uint32_t fat2BeginLBA;
	uint32_t clusterBeginLBA;
	uint8_t sectorsPerCluster;
	uint32_t sectorsPerFat;
	uint32_t rootDirectoryCluster;
} fatUnit;


#define  FAT_FILE_STATUS_OK         0
#define  FAT_FILE_STATUS_OK_DIR     1
#define  FAT_FILE_STATUS_NOT_FOUND  2


typedef struct {
	uint8_t status;
	uint32_t size;
	uint32_t startCluster;
	uint32_t entrySector;
	uint8_t entryOffset;   // 0..15
	char path[128];
} fatFile;


extern void fatShowDirectoryEntries(fatUnit *fat, uint16_t tab, uint32_t cluster);
extern uint32_t fatWrite(fatUnit *fat, fatFile *file, uint32_t offset, uint32_t size, uint8_t *source);
extern uint32_t fatRead(fatUnit *fat, fatFile *file, uint32_t offset, uint32_t size, uint8_t *dest);
extern void fatOpen(fatUnit *fat, fatFile *file, char *path);
extern void fatCreate(fatUnit *fat, fatFile *file, char *folderPath, char *fileName);
extern uint8_t fatDelete(fatUnit *fat, char *path);
extern uint8_t fatMount(fatUnit *fat);


#endif  // __FAT32_H__
